/*
 * Copyright (C) 2005 Alex Murray <murray.alex@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** Contain the functions for operating on the SensorsApplet structure
 *  (represents the applet itself, and its associated variables.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <glib/gprintf.h>
#include "sensors-applet.h"
#include "sensors-applet-gconf-keys.h"
#include "sensors-list-i2c-sys.h"
#include "sensors-list-i2c-proc.h"
#include "sensors-list-i8k.h"
#include "prefs-dialog.h"
#include "about-dialog.h"

#define I2C_BASE_DIR "/sys/devices/"
#define SENSORS_APPLET_MENU_FILE "SensorsApplet.xml"
#define DEFAULT_TIMEOUT 2000

static void prefs_cb(BonoboUIComponent *uic,
		     SensorsApplet    *sensors_applet,
		     const gchar       *verbname) {
	prefs_dialog_show(sensors_applet);
}

static void about_cb(BonoboUIComponent *uic,
		     SensorsApplet    *sensors_applet,
		     const gchar       *verbname) {
	if (sensors_applet->about_dialog) {
		gtk_window_present(GTK_WINDOW(sensors_applet->about_dialog));
		return;
	}
	about_dialog_open(sensors_applet);
}

static void destroy_cb(GtkWidget *widget, SensorsApplet *sensors_applet) {
	/* destory sensors_list and dialogs and finally the applet */
	if (sensors_applet->timeout_id) {
		g_source_remove(sensors_applet->timeout_id);
	}

	GSList *current_sensors_list;

	for (current_sensors_list = sensors_applet->sensors_lists;
	     current_sensors_list != NULL;
	     current_sensors_list = g_slist_next(current_sensors_list)) {
		if (((SensorsList*)(current_sensors_list->data))->list != NULL) {
			gtk_list_store_clear(((SensorsList*)(current_sensors_list->data))->list);
		}
	}

	if (sensors_applet->prefs_dialog != NULL) {
		gtk_widget_destroy(GTK_WIDGET(sensors_applet->prefs_dialog->dialog));
	}
	if (sensors_applet->about_dialog != NULL) {
		gtk_widget_destroy(GTK_WIDGET(sensors_applet->about_dialog));
		}
	gtk_widget_destroy(GTK_WIDGET(sensors_applet->applet));

	g_free(sensors_applet);
	return;
}

static void change_background_cb(PanelApplet *applet, 
				 PanelAppletBackgroundType type,
				 GdkColor *color, 
				 GdkPixmap *pixmap, 
				 SensorsApplet *sensors_applet) {

	g_assert(applet == sensors_applet->applet);
	/* taken from gweather applet */
	GtkRcStyle *rc_style;
	GtkStyle *style;

	/* reset style */
	gtk_widget_set_style(GTK_WIDGET(applet), NULL);
	rc_style = gtk_rc_style_new();
	gtk_widget_modify_style(GTK_WIDGET(applet), rc_style);
	g_object_unref(rc_style);

	switch(type) {
	case PANEL_COLOR_BACKGROUND:
		gtk_widget_modify_bg(GTK_WIDGET(applet),
				     GTK_STATE_NORMAL, color);
		break;

	case PANEL_PIXMAP_BACKGROUND:
		style = gtk_style_copy(GTK_WIDGET(applet)->style);
		if (style->bg_pixmap[GTK_STATE_NORMAL]) {
			g_object_unref(style->bg_pixmap[GTK_STATE_NORMAL]);
		}
		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref(pixmap);
		gtk_widget_set_style(GTK_WIDGET(applet), style);
		break;

	case PANEL_NO_BACKGROUND:
		/* fall through */
	default:
		break;
	}
}

static const BonoboUIVerb sensors_applet_menu_verbs[] = {
	BONOBO_UI_UNSAFE_VERB("Preferences", prefs_cb),
	BONOBO_UI_UNSAFE_VERB("About", about_cb),
	BONOBO_UI_VERB_END
};

static void sensors_applet_setup_sensors(SensorsApplet *sensors_applet) {
	sensors_list_i2c_sys_init(sensors_applet);
	sensors_list_i2c_proc_init(sensors_applet);
	sensors_list_i8k_init(sensors_applet);
}

void sensors_applet_pack_labels(SensorsApplet *sensors_applet) {
	static GtkLabel *no_sensors_enabled_label = NULL;
	int num_active_sensors = 0, num_sensors_per_group, cols, rows, i, j;
	GSList *current_value;
	GSList *current_label;
	GSList *current_sensors_list;

	/* get total number of active sensors - this is the number of
	   values labels that have been created */
	for (current_sensors_list = sensors_applet->sensors_lists;
	     current_sensors_list != NULL;
	     current_sensors_list = g_slist_next(current_sensors_list)) {
		num_active_sensors += g_slist_length(((SensorsList*)(current_sensors_list->data))->values);
	}

	num_sensors_per_group = panel_applet_gconf_get_int(sensors_applet->applet, NUM_SENSORS, NULL);
	if (panel_applet_get_orient(sensors_applet->applet) == PANEL_APPLET_ORIENT_UP || panel_applet_get_orient(sensors_applet->applet) == PANEL_APPLET_ORIENT_DOWN) {
		/* if oriented horizontally, want as many
		   sensors per column as user has defined, then
		   enough columns to hold all the values */
		rows = num_sensors_per_group;
		cols = num_active_sensors / num_sensors_per_group;
		while (rows * cols < num_active_sensors || cols == 0) {
			cols++;
		}
		
	} else {
		/* if oriented vertically, want as many
		   sensors per row as user has defined, then
		   enough rows to hold all the labels */
		cols = num_sensors_per_group;
		rows = num_active_sensors / num_sensors_per_group;
		while (rows * cols < num_active_sensors || rows == 0) {
			rows++;
		}
		
	}
	/* if displaying labels need to modify number of rows / colums
	   to accomodate this */
	if (panel_applet_gconf_get_bool(sensors_applet->applet, SHOW_LABELS, NULL)) {
		if (panel_applet_gconf_get_bool(sensors_applet->applet, LABELS_INLINE, NULL)) {
			/* to display labels next to values need twice
			   as many columns */
			cols *= 2;
		} else {
			/* to display labels above values, we need
			 * twice as many rows as without */
			rows *= 2;
		}
	}

	if (sensors_applet->table == NULL) {
		/* create table and add to applet */
		sensors_applet->table = gtk_table_new(rows, cols, FALSE);
		gtk_table_set_col_spacings(GTK_TABLE(sensors_applet->table), 3);
		gtk_table_set_row_spacings(GTK_TABLE(sensors_applet->table), 0);
		gtk_container_add(GTK_CONTAINER(sensors_applet->applet), sensors_applet->table);
	} else {
		/* remove all children if table already exists so we can start
		 * again */
		GList *children, *current_child;
		children = gtk_container_get_children(GTK_CONTAINER(sensors_applet->table));
		for (current_child = g_list_first(children); current_child != NULL; current_child = g_list_next(current_child)) {
			/* ensure widget does not get destroyed */
			g_object_ref(current_child->data);
			gtk_container_remove(GTK_CONTAINER(sensors_applet->table), GTK_WIDGET(current_child->data));
		}
		/* then resize table */
		gtk_table_resize(GTK_TABLE(sensors_applet->table), rows, cols);
	}
	
	if (num_active_sensors == 0) {
		if (no_sensors_enabled_label == NULL) {
			no_sensors_enabled_label = g_object_new(GTK_TYPE_LABEL,
								"label", _("No sensors enabled!"),
								NULL);
		}
		gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
					  GTK_WIDGET(no_sensors_enabled_label),
					  0, 1,
					  0, 1);
		return;
		
	}
	/* get first SensorsList and values and labels */
	current_sensors_list = sensors_applet->sensors_lists;
	current_value = ((SensorsList*)(current_sensors_list->data))->values;
	current_label = ((SensorsList*)(current_sensors_list->data))->labels;

	/* make sure within this list there are enabled sensors -
	 * otherwise get the next list */
	while (current_label == NULL && current_value == NULL) {
		current_sensors_list = g_slist_next(current_sensors_list);
		if (current_sensors_list != NULL) {
			current_label = ((SensorsList*)(current_sensors_list->data))->labels;
			current_value = ((SensorsList*)(current_sensors_list->data))->values;
		}
	}			

	
	/* pack labels and values into table */
	
	/* if showing labels need to pack these first */
	if (panel_applet_gconf_get_bool(sensors_applet->applet, SHOW_LABELS, NULL)) {
		/* need to first pack labels, then the values either
		   go under the labels or next to them */
		
		/* loop through columns */
		for (i = 0; current_sensors_list != NULL && current_label != NULL && current_value != NULL && i < cols; /* increments depends on how we lay them out - see below */) {
			
			/* loop through rows in a column */
			for (j = 0; current_sensors_list != NULL && current_value != NULL && current_label != NULL && j < rows; /* see bottom of for loop*/) {
				/* attach label at this point */
				gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
							  GTK_WIDGET(current_label->data),
							  i, i + 1,
							  j, j + 1);
				
				/* now attach sensor value to either
				   row below or column next to */
				if (panel_applet_gconf_get_bool(sensors_applet->applet, LABELS_INLINE, NULL)) { /* place value next to label */
					gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
								  GTK_WIDGET(current_value->data),
								  i + 1, i + 2,
								  j, j + 1);
					
					j++;
				} else { /* place value below label */
					gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
								  GTK_WIDGET(current_value->data),
								  i, i + 1,
								  j + 1, j + 2);
					j += 2;
				}
				/* now need to get next label and
				   value from either the
				   current_sensors_list if we still have
				   more, or the next sensors_list that
				   actually has selected values and
				   labels */
				current_label = g_slist_next(current_label);
				current_value = g_slist_next(current_value);
				while (current_label == NULL && current_value == NULL) {
					current_sensors_list = g_slist_next(current_sensors_list);
					if (current_sensors_list != NULL) {
						current_label = ((SensorsList*)(current_sensors_list->data))->labels;
						current_value = ((SensorsList*)(current_sensors_list->data))->values;
						
					} else {
						/* have
						   reached end
						   of
						   sensors_lists
						*/
						break;
					}
				}

			} // end row loop
			/* now increment column index as needed */
			if (panel_applet_gconf_get_bool(sensors_applet->applet, LABELS_INLINE, NULL)) { /* place value next to label */
				i += 2;
			} else {
				i++;
			}
			
			
		} // end column loop	
	} else { /* not showing labels so just pack values */
		for (i = 0; current_value != NULL && i < cols; ++i) {
			for (j = 0; current_value != NULL && j < rows; ++j) {
				gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
							  GTK_WIDGET(current_value->data),
							  i, i + 1,
							  j, j + 1);
				/* now need to get next value from either the
				   current_sensors_list if we still have
				   more, or the next sensors_list that
				   actually has selected values */
				current_value = g_slist_next(current_value);
			      
				while (current_value == NULL) {
					/* are either at the end of
					the values list in the
					current_sensors_list or have
					just got the next sensors_list
					but it has not values in it
					(ie. no sensors enabled) so
					get the next sensors_list
					either way */
					current_sensors_list = g_slist_next(current_sensors_list);
					if (current_sensors_list != NULL) {
						current_value = ((SensorsList*)(current_sensors_list->data))->values;
						
					} else {
						/* have
						   reached end
						   of
						   sensors_lists
						*/
						break;
					}
				}
			}
		}
		
	}
	
} 	    

void sensors_applet_update_display(SensorsApplet *sensors_applet) {
	static gint last_number_of_active_sensors = 0;
	gint number_of_active_sensors = 0;
	GSList *current_value = NULL, *current_label = NULL, *current_sensors_list = NULL;
	
	/* loop through each SensorsList and call its update function
	   to update its own list of labels and values */
	for (current_sensors_list = sensors_applet->sensors_lists;
	     current_sensors_list != NULL;
	     current_sensors_list = g_slist_next(current_sensors_list)) {
		((SensorsList*)(current_sensors_list->data))->update_labels(((SensorsList*)(current_sensors_list->data)));

		/* now get number of active sensors */
		number_of_active_sensors += g_slist_length(((SensorsList*)(current_sensors_list->data))->values);
	}
	if (number_of_active_sensors != last_number_of_active_sensors) {
		/* now repack labels into table update display of applets */
		sensors_applet_pack_labels(sensors_applet);
	}
	last_number_of_active_sensors = number_of_active_sensors;

	gtk_widget_show_all(GTK_WIDGET(sensors_applet->applet));
}

static void sensors_applet_write_defaults_to_gconf(SensorsApplet *sensors_applet) {
	panel_applet_gconf_set_bool(sensors_applet->applet, FAHRENHEIT, FALSE, NULL);
	panel_applet_gconf_set_int(sensors_applet->applet, NUM_SENSORS, 1, NULL);
	panel_applet_gconf_set_bool(sensors_applet->applet, SHOW_LABELS, TRUE, NULL);
	panel_applet_gconf_set_bool(sensors_applet->applet, LABELS_INLINE, TRUE, NULL);
	panel_applet_gconf_set_bool(sensors_applet->applet, SHOW_UNITS, TRUE, NULL);
	panel_applet_gconf_set_int(sensors_applet->applet, TIMEOUT, DEFAULT_TIMEOUT, NULL);
	panel_applet_gconf_set_int(sensors_applet->applet, FONT_SIZE, MEDIUM, NULL);
	panel_applet_gconf_set_bool(sensors_applet->applet, IS_SETUP, FALSE, NULL);

}

void sensors_applet_init(SensorsApplet *sensors_applet) {
	SensorsList *sys_sensors_list;
	
	g_assert(sensors_applet);
	g_assert(sensors_applet->applet);
	
	panel_applet_set_flags(sensors_applet->applet, PANEL_APPLET_EXPAND_MINOR);
	panel_applet_setup_menu_from_file(sensors_applet->applet,
					  NULL,
					  SENSORS_APPLET_MENU_FILE,
					  NULL,
					  sensors_applet_menu_verbs,
					  sensors_applet);

	g_signal_connect(sensors_applet->applet, 
			 "change_background",
			 G_CALLBACK(change_background_cb), 
			 sensors_applet);

	g_signal_connect(sensors_applet->applet, "destroy",
			 G_CALLBACK(destroy_cb),
			 sensors_applet);

	/* if not setup, write defaults to gconf */
	if (!panel_applet_gconf_get_bool(sensors_applet->applet, IS_SETUP, NULL)) {
		sensors_applet_write_defaults_to_gconf(sensors_applet);
	}

	sensors_applet_setup_sensors(sensors_applet);
	/* should have created sensors_lists above, but if have
	   not was because we couldn't find any sensors */
	if (sensors_applet->sensors_lists == NULL) {
		GtkWidget *label;	
		label = gtk_label_new(_("No sensors found!"));
		gtk_container_add(GTK_CONTAINER(sensors_applet->applet), label);
		gtk_widget_show_all(GTK_WIDGET(sensors_applet->applet));
		sensors_applet->prefs_dialog = g_new0(PrefsDialog, 1);
		sensors_applet->prefs_dialog->sensors_applet = sensors_applet;
		prefs_dialog_init(sensors_applet->prefs_dialog);
		return;
	}
	
	sensors_applet->prefs_dialog = g_new0(PrefsDialog, 1);
	sensors_applet->prefs_dialog->sensors_applet = sensors_applet;
	prefs_dialog_init(sensors_applet->prefs_dialog);

	sensors_applet->tooltips = gtk_tooltips_new();
	gtk_tooltips_set_tip(sensors_applet->tooltips, GTK_WIDGET(sensors_applet->applet), _("Sensors Applet"), "");

	sensors_applet_update_display(sensors_applet);
	sensors_applet_pack_labels(sensors_applet);
	sensors_applet->timeout_id = g_timeout_add(panel_applet_gconf_get_int(sensors_applet->applet, TIMEOUT, NULL), (GSourceFunc)sensors_applet_update_display, sensors_applet);

	gtk_widget_show_all(GTK_WIDGET(sensors_applet->applet));
}



