/*
 * Copyright (C) 2005 Alex Murray <murray.alex@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "sensors-applet-gconf-keys.h"
#include "sensors-list-i2c-sys.h"
#include "sensors-list.h"

#define I2C_SYS_BASE_DIR "/sys"
#define ID "i2c-sys"

static SensorsList *sensors_list_i2c_sys_sensors_list_new(SensorsApplet *sensors_applet) {
	SensorsList *sys_sensors_list;
	sys_sensors_list = g_new0(SensorsList, 1);
	sys_sensors_list->sensors_applet = sensors_applet;
	sys_sensors_list->id = ID;
	/* set update and error functions */
	sys_sensors_list->update_labels = sensors_list_i2c_sys_update_labels;
	sensors_list_init(sys_sensors_list);

	sensors_applet->sensors_lists = g_slist_append(sensors_applet->sensors_lists, sys_sensors_list);
	return sys_sensors_list;
}


static void sensors_list_i2c_sys_setup_from_gconf(SensorsApplet *sensors_applet) {
	SensorsList *sys_sensors_list = sensors_list_i2c_sys_sensors_list_new(sensors_applet);
	sensors_list_setup_from_gconf(sys_sensors_list);
}
	
static void sensors_list_i2c_sys_add_sensor(SensorsApplet *sensors_applet, gchar *path) {
	static SensorsList *sys_sensors_list = NULL;
	gchar *filename;
	gchar *label;
	gboolean enable;
	guint sensor_type;

	GtkTreeIter iter;
	if (sys_sensors_list == NULL) {
		sys_sensors_list = sensors_list_i2c_sys_sensors_list_new(sensors_applet);
		 
	}

	filename = g_path_get_basename(path);

	/* setup temp2 as CPU sensor and enable it */
	if (g_ascii_strcasecmp(filename, "temp2_input") == 0) {
		label = _("CPU");
		sensor_type = I2C_SYS_TEMP_SENSOR;
		enable = TRUE;
	} else {
		switch(filename[0]) {
		case 'c':     /* currents are curr?_input */
			label = g_strndup(filename, 5);
			sensor_type = I2C_SYS_CURRENT_SENSOR;
			break;			
		case 'f':     /* fans are "fan?_input" */
			label = g_strndup(filename, 4);
			sensor_type = I2C_SYS_FAN_SENSOR;
			break;
		case 't':     /* temps are "temp?_input" */
			label = g_strndup(filename, 5);
			sensor_type = I2C_SYS_TEMP_SENSOR;
			break;
		case 'i':     /* voltages are "in?_input" */
			label = g_strndup(filename, 3);
			sensor_type = I2C_SYS_VOLTAGE_SENSOR;
			break;
		default:
			/* SHOULDN'T BE ABLE
			 * TO GET HERE!! */
			g_assert_not_reached();
		}
		/* disable all other sensors */
		enable = FALSE;
	}
	gtk_list_store_append(sys_sensors_list->list, &iter);
	gtk_list_store_set(sys_sensors_list->list,
			   &iter,
			   PATH_COLUMN, path,
			   FILENAME_COLUMN, filename,
			   LABEL_COLUMN, label,
			   ENABLE_COLUMN, enable,
			   ALARM_VALUE_COLUMN, 0.0,
			   ALARM_TYPE_COLUMN, ALARM_WHEN_VALUE_GREATER_THAN_THRESHOLD,
			   ALARM_ENABLE_COLUMN, FALSE,
			   ALARM_COMMAND_COLUMN, "",
			   ALARM_TIMEOUT_COLUMN, 0,
			   ALARM_TIMEOUT_INDEX_COLUMN, -1,
			   SENSOR_TYPE_COLUMN, sensor_type,
			   MULTIPLIER_COLUMN, 1.0,
			   OFFSET_COLUMN, 0.0,
			   -1);
}	

/* recursive function to find sensors in a given path - once found
   will call function to add sensor to the sensors_list */
static void sensors_list_i2c_sys_find_sensors(SensorsApplet *sensors_applet, const gchar *path) {
	GDir *dir;
	FILE *fp;
	const gchar* new_file, *new_path;
	if (g_file_test(path, G_FILE_TEST_IS_REGULAR)) {
		if(g_strrstr(path, "_input")) {
			/* also test can actually open file for
			   reading */
			if (fp = fopen(path, "r")) {
				fclose(fp);
				sensors_list_i2c_sys_add_sensor(sensors_applet, g_strdup(path));
			}
		}
		
	}
	/* if is a directory (but not a symlinked dir as this
	   will lead us in circular loops) descend into it and look
	   for a sensor dir
	*/
	if (g_file_test(path, G_FILE_TEST_IS_DIR) && !g_file_test(path, G_FILE_TEST_IS_SYMLINK)) {
		dir = g_dir_open(path, 0, NULL);
		if (dir != NULL) {
			while(new_file = g_dir_read_name(dir)) {
				new_path = g_strdup_printf("%s/%s", path, new_file);
				sensors_list_i2c_sys_find_sensors(sensors_applet, new_path);
			}
			g_dir_close(dir);
		}
	}
}

/* to be called to setup for sys sensors */
void sensors_list_i2c_sys_init(SensorsApplet *sensors_applet) {
	/* if not setup find sensors manually */
	if (!panel_applet_gconf_get_bool(sensors_applet->applet, ID "/" IS_SETUP, NULL)) {
		/* call function to recursively look for sensors
		   starting at the defined base directory */
		sensors_list_i2c_sys_find_sensors(sensors_applet, I2C_SYS_BASE_DIR);
	} else {
		if (sensors_list_gconf_paths_valid(sensors_applet, ID)) {
			sensors_list_i2c_sys_setup_from_gconf(sensors_applet);
		} else {
			/* alert user to error and restart from
			   scratch */
			panel_applet_gconf_set_bool(sensors_applet->applet, ID "/" IS_SETUP, FALSE, NULL);
			sensors_list_gconf_error_occurred(ID);
			sensors_list_i2c_sys_find_sensors(sensors_applet, I2C_SYS_BASE_DIR);
		}
	}
}



void sensors_list_i2c_sys_update_labels(SensorsList *sensors_list) {
	/* ensure we have a data structure to work from */
	g_assert(sensors_list != NULL);
	g_assert(sensors_list->list != NULL);

	/* to iterate through the sensors data structure
	   (sensors_list) */
	GtkTreeIter iter;
	gboolean not_end_of_list = TRUE;
	gchar *sensor_path = NULL;
	gchar *sensor_filename = NULL;
	gchar *sensor_label = NULL;
	gboolean sensor_enabled;
	gdouble sensor_alarm_value, sensor_multiplier, sensor_offset;
	gboolean sensor_alarm_enabled;
	AlarmType sensor_alarm_type;
	I2cSysSensorType sensor_type;

	/* to open and access the value of each sensor */
	FILE *fp;
	gfloat sensor_value;

	/* to build the list of labels as we go */
	gchar *label_text = NULL;
	gchar *value_text = NULL;
	gchar *font_size = NULL;

	GSList *current_label = NULL;
	GSList *current_value = NULL;

	/* get the first label in the list */
	current_label = sensors_list->labels;
	current_value = sensors_list->values;
	
	/* now step through the GtkListStore sensors_list to
	   find which sensors are enabled */
	for (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(sensors_list->list), &iter); not_end_of_list; not_end_of_list = gtk_tree_model_iter_next(GTK_TREE_MODEL(sensors_list->list), &iter)) {
		gtk_tree_model_get(GTK_TREE_MODEL(sensors_list->list), 
				   &iter,
				   PATH_COLUMN, &sensor_path,
				   FILENAME_COLUMN, &sensor_filename,
				   LABEL_COLUMN, &sensor_label,
				   ENABLE_COLUMN, &sensor_enabled,
				   ALARM_VALUE_COLUMN, &sensor_alarm_value,
				   ALARM_TYPE_COLUMN, &sensor_alarm_type,
				   ALARM_ENABLE_COLUMN, &sensor_alarm_enabled,
				   SENSOR_TYPE_COLUMN, &sensor_type,
				   MULTIPLIER_COLUMN, &sensor_multiplier,
				   OFFSET_COLUMN, &sensor_offset,
				   -1);

		if (sensor_enabled) {
			g_assert(fp = fopen(sensor_path, "r"));
			/* read in value from sensor file - will be
			   millidegrees, millivolts or rpm - as
			   integers */
			fscanf(fp, "%f", &sensor_value);
			fclose(fp);

			switch (sensor_type) {
			case I2C_SYS_TEMP_SENSOR: 
				sensor_value /= 1000.0;
				if (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, FAHRENHEIT, NULL)) {
					/* Convert to fahrenheit */
					sensor_value = 32.0 + 9.0 * sensor_value / 5.0;
					sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
					value_text = g_strdup_printf("%3.1f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_UNITS, NULL) ? "\302\260F" : ""));
				} else {
					sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
					value_text = g_strdup_printf("%3.1f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_UNITS, NULL) ? "\302\260C" : ""));
				}
				break;
				
			case I2C_SYS_FAN_SENSOR:
				sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
				value_text = g_strdup_printf("%4.0f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_UNITS, NULL) ? _("RPM") : ""));
				break;
				
			case I2C_SYS_VOLTAGE_SENSOR:
				sensor_value /= 1000.0;
				sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
				value_text = g_strdup_printf("%4.2f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_UNITS, NULL) ? _("V") : ""));
				break;
				
			case I2C_SYS_CURRENT_SENSOR:
				sensor_value /= 1000.0;
				sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
				value_text = g_strdup_printf("%4.2f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_UNITS, NULL) ? _("A") : ""));
				break;
				
			default:
				g_assert_not_reached();	
		
			} // end switch(sensor_type)
			gchar *old_value_text = value_text;

			/* get font size */
			font_size = sensors_list_get_font_size(sensors_list);

			if (sensor_alarm_enabled) {
				switch (sensor_alarm_type) {
				case ALARM_WHEN_VALUE_GREATER_THAN_THRESHOLD:
					      if (sensor_value >= sensor_alarm_value) {
						      /* make value
						       * text red with
						       correct font
						       size and do
						       alarm command */
						      value_text = g_markup_printf_escaped("<span foreground=\"#FF0000\" size=\"%s\">%s</span>", font_size, old_value_text);
						      sensors_list_alarm_on(sensors_list, &iter);
					      } else {
						      /* just do font size */
						      value_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, old_value_text);
						      sensors_list_alarm_off(sensors_list, &iter);
					      }				
				break;
				case ALARM_WHEN_VALUE_LESS_THAN_THRESHOLD:
					      if (sensor_value <= sensor_alarm_value) {
						      /* make value
						       * text red with
						       correct font
						       size and do
						       alarm command */
						      value_text = g_markup_printf_escaped("<span foreground=\"#FF0000\" size=\"%s\">%s</span>", font_size, old_value_text);
						      sensors_list_alarm_on(sensors_list, &iter);
					      } else {
						      /* just do font size */
						      value_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, old_value_text);
						      sensors_list_alarm_off(sensors_list, &iter);
					      }				
				break;
				default: 
					g_assert_not_reached();
				} // end switch(sensor_alarm_type)
			} else {
				/* just do font size */
				value_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, old_value_text);
				sensors_list_alarm_off(sensors_list, &iter);
			}

			g_free(old_value_text);
			sensors_list_add_label_to_list(&(sensors_list->values), &current_value, value_text);
			g_free(value_text);
			
			/* advance to next element in list */
			current_value = g_slist_next(current_value);
			
			/* if have labels enabled, also add label to
			 * labels list */
			if (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_LABELS, NULL)) {
				/* create label text with font size
				 * formatting */
				label_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, sensor_label);
				sensors_list_add_label_to_list(&(sensors_list->labels), &current_label, label_text);
				g_free(label_text);
				current_label = g_slist_next(current_label);
			}
			/* free allocated string from font size */
			g_free(font_size);
		} // end if(sensor_enabled)
		g_free(sensor_path);
		g_free(sensor_filename);
		g_free(sensor_label);
	}
	
	/* now remove any remaining elements in either list to free up
	   memory */
	sensors_list_crop_list(&(sensors_list->values), &current_value);
	sensors_list_crop_list(&(sensors_list->labels), &current_label);
	
	
}
