/*
 * Copyright (C) 2005 Alex Murray <murray.alex@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "sensors-applet-gconf-keys.h"
#include "sensors-list-i8k.h"
#include "sensors-list.h"

#define I8K_DEVICE_PATH "/proc/i8k"
#define I8K_DEVICE_FILE "i8k"
#define ID "i8k"

static SensorsList *sensors_list_i8k_sensors_list_new(SensorsApplet *sensors_applet) {
	SensorsList *i8k_sensors_list;
	i8k_sensors_list = g_new0(SensorsList, 1);
	i8k_sensors_list->sensors_applet = sensors_applet;
	i8k_sensors_list->id = ID;
	/* set update function */
	i8k_sensors_list->update_labels = sensors_list_i8k_update_labels;

	sensors_list_init(i8k_sensors_list);

	sensors_applet->sensors_lists = g_slist_append(sensors_applet->sensors_lists, i8k_sensors_list);
	return i8k_sensors_list;
}

static void sensors_list_i8k_setup_from_gconf(SensorsApplet *sensors_applet) {
	SensorsList *i8k_sensors_list = sensors_list_i8k_sensors_list_new(sensors_applet);
	sensors_list_setup_from_gconf(i8k_sensors_list);
}

static void sensors_list_i8k_setup_manually(SensorsApplet *sensors_applet) {
	SensorsList *i8k_sensors_list;
	GtkTreeIter iter;

	if (g_file_test(I8K_DEVICE_PATH, G_FILE_TEST_EXISTS)) {
		/* i8k stuff exists so go setup manually */
		i8k_sensors_list = sensors_list_i8k_sensors_list_new(sensors_applet);
		/* with i8k have only 3 fixed sensors, all accessed
		   from the I8K_DEVICE_PATH */
		gtk_list_store_append(i8k_sensors_list->list, &iter);
		gtk_list_store_set(i8k_sensors_list->list,
				   &iter,
				   PATH_COLUMN, I8K_DEVICE_PATH,
				   FILENAME_COLUMN, I8K_DEVICE_FILE,
				   LABEL_COLUMN, _("CPU"),
				   ENABLE_COLUMN, TRUE,
				   ALARM_VALUE_COLUMN, 0.0,
				   ALARM_TYPE_COLUMN, ALARM_WHEN_VALUE_GREATER_THAN_THRESHOLD,
				   ALARM_ENABLE_COLUMN, FALSE,
				   ALARM_COMMAND_COLUMN, "",
				   ALARM_TIMEOUT_COLUMN, 0,
				   ALARM_TIMEOUT_INDEX_COLUMN, -1,
				   SENSOR_TYPE_COLUMN, I8K_CPU_SENSOR,
				   MULTIPLIER_COLUMN, 1.0,
				   OFFSET_COLUMN, 0.0,
				   -1);

		gtk_list_store_append(i8k_sensors_list->list, &iter);
		gtk_list_store_set(i8k_sensors_list->list,
				   &iter,
				   PATH_COLUMN, I8K_DEVICE_PATH,
				   FILENAME_COLUMN, I8K_DEVICE_FILE,
				   LABEL_COLUMN, _("FAN 1"),
				   ENABLE_COLUMN, FALSE,
				   ALARM_VALUE_COLUMN, 0.0,
				   ALARM_TYPE_COLUMN, ALARM_WHEN_VALUE_GREATER_THAN_THRESHOLD,
				   ALARM_ENABLE_COLUMN, FALSE,
				   ALARM_COMMAND_COLUMN, "",
				   ALARM_TIMEOUT_COLUMN, 0,
				   ALARM_TIMEOUT_INDEX_COLUMN, -1,
				   SENSOR_TYPE_COLUMN, I8K_FAN1_SENSOR,
				   MULTIPLIER_COLUMN, 1.0,
				   OFFSET_COLUMN, 0.0,
				   -1);

		gtk_list_store_append(i8k_sensors_list->list, &iter);
		gtk_list_store_set(i8k_sensors_list->list,
				   &iter,
				   PATH_COLUMN, I8K_DEVICE_PATH,
				   FILENAME_COLUMN, I8K_DEVICE_FILE,
				   LABEL_COLUMN, _("FAN 2"),
				   ENABLE_COLUMN, FALSE,
				   ALARM_VALUE_COLUMN, 0.0,
				   ALARM_TYPE_COLUMN, ALARM_WHEN_VALUE_GREATER_THAN_THRESHOLD,
				   ALARM_ENABLE_COLUMN, FALSE,
				   ALARM_COMMAND_COLUMN, "",
				   ALARM_TIMEOUT_COLUMN, 0,
				   ALARM_TIMEOUT_INDEX_COLUMN, -1,
				   SENSOR_TYPE_COLUMN, I8K_FAN2_SENSOR,
				   MULTIPLIER_COLUMN, 1.0,
				   OFFSET_COLUMN, 0.0,
				   -1);

	}
}
/* to be called externally to setup for i8k sensors */
void sensors_list_i8k_init(SensorsApplet *sensors_applet) {
	/* if not setup write default values to gconf and try to find
	 * sensors manually */
	if (!panel_applet_gconf_get_bool(sensors_applet->applet, ID "/" IS_SETUP, NULL)) {
		sensors_list_i8k_setup_manually(sensors_applet);
	} else {
		if (sensors_list_gconf_paths_valid(sensors_applet, ID)) {
			sensors_list_i8k_setup_from_gconf(sensors_applet);
		} else {
			/* restart from scratch */
			panel_applet_gconf_set_bool(sensors_applet->applet, ID "/" IS_SETUP, FALSE, NULL);
			sensors_list_gconf_error_occurred(ID);
			sensors_list_i8k_setup_manually(sensors_applet);
		}
	}
}
	
void sensors_list_i8k_update_labels(SensorsList *sensors_list) {
	/* ensure we have a data structure to work from */
	g_assert(sensors_list != NULL);
	g_assert(sensors_list->list != NULL);

	/* to iterate through the sensors data structure
	   (sensors_list) */
	GtkTreeIter iter;
	gboolean not_end_of_list = TRUE;
	gchar *sensor_path = NULL;
	gchar *sensor_filename = NULL;
	gchar *sensor_label = NULL;
	gboolean sensor_enabled;
	gdouble sensor_alarm_value, sensor_multiplier, sensor_offset;
	gboolean sensor_alarm_enabled;
	AlarmType sensor_alarm_type;
	I8kSensorType sensor_type;

	/* to open and access the value of each sensor */
	FILE *fp;
	GError *error = NULL;
	gint cpu_temp, fan1_status, fan2_status, fan1_rpm, fan2_rpm;
	gint sensor_value;
	gint space_count;

	/* to build the list of labels as we go */
	gchar *label_text = NULL;
	gchar *value_text = NULL;
	gchar *font_size = NULL;

	GSList *current_label = NULL;
	GSList *current_value = NULL;

	/* get the first label in the list */
	current_label = sensors_list->labels;
	current_value = sensors_list->values;
	
	/* now step through the GtkListStore sensors_list to
	   find which sensors are enabled */
	for (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(sensors_list->list), &iter); not_end_of_list; not_end_of_list = gtk_tree_model_iter_next(GTK_TREE_MODEL(sensors_list->list), &iter)) {
		gtk_tree_model_get(GTK_TREE_MODEL(sensors_list->list), 
				   &iter,
				   PATH_COLUMN, &sensor_path,
				   FILENAME_COLUMN, &sensor_filename,
				   LABEL_COLUMN, &sensor_label,
				   ENABLE_COLUMN, &sensor_enabled,
				   ALARM_VALUE_COLUMN, &sensor_alarm_value,
				   ALARM_TYPE_COLUMN, &sensor_alarm_type,
				   ALARM_ENABLE_COLUMN, &sensor_alarm_enabled,
				   SENSOR_TYPE_COLUMN, &sensor_type,
				   MULTIPLIER_COLUMN, &sensor_multiplier,
				   OFFSET_COLUMN, &sensor_offset,
				   -1);

		if (sensor_enabled) {
			fp = fopen(sensor_path, "r");
			space_count = 0;

			while (space_count < 3) {
				if (fgetc(fp) == ' ') {
					space_count++;
				}
			}
			fscanf(fp, "%d %d %d %d %d", &cpu_temp, &fan1_status, &fan2_status, &fan1_rpm, &fan2_rpm);
			fclose(fp);
			switch (sensor_type) {
			case I8K_CPU_SENSOR:
				sensor_value = cpu_temp;
				if (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, FAHRENHEIT, NULL)) {
					/* Convert to fahrenheit */
					sensor_value = (gint)32.0 + 9.0 * sensor_value / 5.0;
					sensor_value = ((gint)((sensor_value * sensor_multiplier) + sensor_offset));

					value_text = g_strdup_printf("%3d%s", sensor_value, (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_UNITS, NULL) ? "\302\260F" : ""));
				} else {
					sensor_value = ((gint)((sensor_value * sensor_multiplier) + sensor_offset));
					value_text = g_strdup_printf("%2d%s", sensor_value, (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_UNITS, NULL) ? "\302\260C" : ""));
				}					
				break;
			case I8K_FAN1_SENSOR:
				sensor_value = fan1_rpm;
				sensor_value = ((gint)((sensor_value * sensor_multiplier) + sensor_offset));
				value_text = g_strdup_printf("%4d%s", sensor_value, (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_UNITS, NULL) ? _("RPM") : ""));

				break;

			case I8K_FAN2_SENSOR:
				sensor_value = fan2_rpm;
				sensor_value = ((gint)((sensor_value * sensor_multiplier) + sensor_offset));
				value_text = g_strdup_printf("%4d%s", sensor_value, (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_UNITS, NULL) ? _("RPM") : ""));
				break;

			default:
				g_assert_not_reached();			
			} // end switch (sensor_type)
			gchar *old_value_text = value_text;

			/* get font size */
			font_size = sensors_list_get_font_size(sensors_list);

			if (sensor_alarm_enabled) {
				switch (sensor_alarm_type) {
				case ALARM_WHEN_VALUE_GREATER_THAN_THRESHOLD:
					if (sensor_value >= sensor_alarm_value) {
						/* make value
						 * text red with
						 correct font
						 size and do
						 alarm command */
						value_text = g_markup_printf_escaped("<span foreground=\"#FF0000\" size=\"%s\">%s</span>", font_size, old_value_text);
						sensors_list_alarm_on(sensors_list, &iter);
					} else {
						/* just do font size */
						value_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, old_value_text);
						sensors_list_alarm_off(sensors_list, &iter);
					}				
					break;
				case ALARM_WHEN_VALUE_LESS_THAN_THRESHOLD:
					if (sensor_value <= sensor_alarm_value) {
						/* make value
						 * text red with
						 correct font
						 size and do
						 alarm command */
						value_text = g_markup_printf_escaped("<span foreground=\"#FF0000\" size=\"%s\">%s</span>", font_size, old_value_text);
						sensors_list_alarm_on(sensors_list, &iter);
					} else {
						/* just do font size */
						value_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, old_value_text);
						sensors_list_alarm_off(sensors_list, &iter);
					}				
					break;
				default: 
					g_assert_not_reached();
				} // end switch(sensor_alarm_type)
			} else {
				/* just do font size */
				value_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, old_value_text);
				sensors_list_alarm_off(sensors_list, &iter);
			}
			g_free(old_value_text);
			sensors_list_add_label_to_list(&(sensors_list->values), &current_value, value_text);
			g_free(value_text);
			

			/* advance to next element in list */
			current_value = g_slist_next(current_value);
			
			/* if have labels enabled, also add label to
			 * labels list */
			if (panel_applet_gconf_get_bool(sensors_list->sensors_applet->applet, SHOW_LABELS, NULL)) {
				/* create label text with font size
				 * formatting */
				label_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, sensor_label);
				sensors_list_add_label_to_list(&(sensors_list->labels), &current_label, label_text);
				g_free(label_text);
				current_label = g_slist_next(current_label);
			}

			g_free(font_size);
		}
		g_free(sensor_path);
		g_free(sensor_filename);
		g_free(sensor_label);
	}
	
	/* now remove any remaining elements in either list to free up
	   memory */
	sensors_list_crop_list(&(sensors_list->values), &current_value);
	sensors_list_crop_list(&(sensors_list->labels), &current_label);
	
}

